#ifndef SVN_VERSION_H
#define SVN_VERSION_H

#define SVN_VERSION "ae68f3290ac82d08b4d9767acfe0daaccaeeb883 [/srcdest/boinc] (archlinux:/build/boinc/src/boinc [makepkg]) [Server-Release: server_release/1.x/1.x.0-3446-gae68f3290ac82d08b4d9767acfe0daaccaeeb883]"
#define GIT_REVISION 0xae68f329
#define GIT_DATE 1777831803
#define REPOSITORY__SRCDEST_BOINC 1

#endif
